"""Hybrid structure management for Obsidian vault operations."""

import re
from pathlib import Path
from typing import Any

from ..core.config import CKCConfig
from ..core.hybrid_config import (
    DirectoryTier,
    HybridStructureConfig,
    NumberingSystem,
    NumberManagerConfig,
)
from ..core.metadata import KnowledgeMetadata, MetadataManager
from .obsidian import ObsidianVaultManager


class StructureManager:
    """Manages hybrid structure creation and validation."""

    def __init__(self, hybrid_config: HybridStructureConfig):
        self.config = hybrid_config
        self.number_manager = NumberManagerConfig(
            system_type=hybrid_config.numbering_system, step_size=10, max_number=990
        )

    def get_vault_structure(self) -> dict[str, str]:
        """Get vault structure based on configuration."""
        if self.config.custom_structure:
            return self._flatten_custom_structure()
        else:
            return self._flatten_default_structure()

    def _flatten_default_structure(self) -> dict[str, str]:
        """Flatten default structure into path -> description mapping."""
        structure = self.config.get_default_structure()
        flattened = {}

        # Add all tier directories
        for _tier_name, tier_dirs in structure.items():
            for dir_name, description in tier_dirs.items():
                flattened[dir_name] = description

        # Add Knowledge_Base subdirectories if using ten-step system
        if self.config.numbering_system == NumberingSystem.TEN_STEP:
            kb_structure = self.config.get_knowledge_base_structure()
            # Get the actual Knowledge_Base structure
            if "20_Knowledge_Base" in kb_structure:
                kb_content = kb_structure["20_Knowledge_Base"]
                if "subdirectories" in kb_content:
                    self._add_subdirectories(
                        flattened, kb_content["subdirectories"], "20_Knowledge_Base"
                    )

        return flattened

    def _flatten_custom_structure(self) -> dict[str, str]:
        """Flatten custom structure into path -> description mapping."""
        flattened = {}

        if self.config.custom_structure is not None:
            for _tier_name, tier_dirs in self.config.custom_structure.items():
                for dir_name, description in tier_dirs.items():
                    flattened[dir_name] = description

        return flattened

    def _add_subdirectories(
        self, flattened: dict, structure: dict, base_path: str = ""
    ) -> None:
        """Recursively add subdirectories to flattened structure."""
        for key, value in structure.items():
            current_path = f"{base_path}/{key}" if base_path else key

            if isinstance(value, dict):
                if "subdirs" in value:
                    # This is a directory with simple subdirectories
                    for subdir, desc in value["subdirs"].items():
                        subdir_path = f"{current_path}/{subdir}"
                        flattened[subdir_path] = desc
                elif "description" in value and not any(
                    x in value for x in ["subdirs", "subdirectories"]
                ):
                    # This is a simple directory with description
                    flattened[current_path] = value["description"]
                else:
                    # This is a nested structure - recurse
                    self._add_subdirectories(flattened, value, current_path)
            else:
                # This is a simple directory with string description
                flattened[current_path] = value


class KnowledgeClassifier:
    """Classifies knowledge content for automatic placement."""

    def __init__(self, hybrid_config: HybridStructureConfig):
        self.config = hybrid_config

        # Classification patterns
        self.tech_keywords = {
            "python": ["python", "pip", "conda", "pytest", "django", "flask", "pandas"],
            "javascript": [
                "javascript",
                "js",
                "node",
                "npm",
                "react",
                "vue",
                "angular",
            ],
            "typescript": ["typescript", "ts", "tsx", "type", "interface"],
            "shell": ["bash", "shell", "sh", "zsh", "fish", "terminal", "command"],
            "docker": ["docker", "dockerfile", "container", "image", "compose"],
            "git": [
                "git",
                "github",
                "gitlab",
                "commit",
                "branch",
                "merge",
                "pull request",
            ],
        }

        self.claude_models = ["opus", "sonnet", "haiku", "claude"]

    def classify_content(
        self,
        content: str,
        metadata: KnowledgeMetadata,
        source_path: Path,
        project_name: str | None = None,
    ) -> str:
        """Classify content and return target directory path."""

        # SHARED KNOWLEDGE FIRST: Category/Tag-based classification takes priority
        # Always classify shared knowledge (prompt, code, concept, resource)
        # regardless of project

        # Phase 1: Category metadata takes absolute priority
        category = (
            getattr(metadata, "category", "").lower()
            if hasattr(metadata, "category") and metadata.category
            else ""
        )

        if category == "concept":
            return self._classify_concept(content, metadata)
        elif category == "code":
            return self._classify_code(content, metadata)
        elif category == "prompt":
            return self._classify_prompt(content, metadata)
        elif category == "resource":
            return self._classify_resource(content, metadata)
        elif category == "project_log":
            # Only project_log category goes to Projects
            effective_project = project_name or getattr(metadata, "project", None)
            return (
                f"10_Projects/{effective_project}"
                if effective_project
                else "10_Projects"
            )

        # Phase 2: Tag-based classification as fallback
        # More specific tags take precedence over general ones
        if "command" in metadata.tags:
            return self._classify_command(content, metadata)
        elif "code" in metadata.tags:
            return self._classify_code(content, metadata)
        elif "concept" in metadata.tags:
            return self._classify_concept(content, metadata)
        elif "resource" in metadata.tags:
            return self._classify_resource(content, metadata)
        elif "prompt" in metadata.tags:
            return self._classify_prompt(content, metadata)
        elif "project_log" in metadata.tags:
            # Only project_log tag goes to Projects
            effective_project = project_name or getattr(metadata, "project", None)
            return (
                f"10_Projects/{effective_project}"
                if effective_project
                else "10_Projects"
            )

        # Phase 3: Project-specific classification (only for uncategorized content)
        effective_project = project_name or getattr(metadata, "project", None)
        if effective_project:
            return f"10_Projects/{effective_project}"

        # Phase 4: Default classification based on content analysis
        return self._classify_by_content_analysis(content, metadata)

    def _classify_prompt(self, content: str, metadata: KnowledgeMetadata) -> str:
        """Classify prompt content with improved logic."""
        base_path = "20_Knowledge_Base/Prompts"

        # Priority 1: Check for explicit subcategory in metadata
        subcategory = getattr(metadata, "subcategory", None)
        if subcategory:
            subcategory_mapping = {
                "templates": "Templates",
                "best_practices": "Best_Practices",
                "improvement_log": "Improvement_Log",
                "domain_specific": "Domain_Specific",
            }
            mapped_subcategory = subcategory_mapping.get(
                subcategory.lower(), subcategory
            )
            return f"{base_path}/{mapped_subcategory}"

        # Priority 2: Best practices (high success rate or production status)
        if (
            metadata.success_rate and metadata.success_rate >= 80
        ) or metadata.status == "production":
            return f"{base_path}/Best_Practices"

        # Improvement log - STRICT criteria for prompt improvement tracking
        if self._is_improvement_record(content, metadata):
            return f"{base_path}/Improvement_Log"

        # Templates - generic reusable prompts
        if self._is_template_prompt(content, metadata):
            return f"{base_path}/Templates"

        # For simple test cases or when no specific classification criteria are met,
        # place files in the base Prompts directory rather than subdirectories
        return base_path

    def _is_improvement_record(self, content: str, metadata: KnowledgeMetadata) -> bool:
        """Check if content is specifically a prompt improvement record."""
        content_lower = content.lower()

        # Must explicitly mention improvement concepts
        improvement_keywords = [
            "improvement",
            "version",
            "iteration",
            "refined",
            "updated",
            "optimized",
            "enhanced",
            "modified",
            "revised",
            "evolved",
        ]

        # Must have version tracking or comparison content
        version_indicators = [
            "v1",
            "v2",
            "version 1",
            "version 2",
            "before/after",
            "original",
            "improved",
            "comparison",
            "performance",
        ]

        has_improvement = any(
            keyword in content_lower for keyword in improvement_keywords
        )
        has_version_tracking = any(
            indicator in content_lower for indicator in version_indicators
        )

        return has_improvement and has_version_tracking

    def _is_template_prompt(self, content: str, metadata: KnowledgeMetadata) -> bool:
        """Check if content is a template prompt."""
        content_lower = content.lower()

        template_indicators = [
            "template",
            "reusable",
            "general",
            "generic",
            "placeholder",
            "[insert",
            "{variable}",
            "customize",
            "adapt",
        ]

        return any(indicator in content_lower for indicator in template_indicators)

    def _classify_code(self, content: str, metadata: KnowledgeMetadata) -> str:
        """Classify code content."""
        base_path = "20_Knowledge_Base/Code_Snippets"

        # Priority 1: Check for explicit subcategory (language) in metadata
        subcategory = getattr(metadata, "subcategory", None)
        if subcategory:
            language_mapping = {
                "python": "Python",
                "javascript": "JavaScript",
                "typescript": "TypeScript",
                "shell": "Shell",
                "bash": "Shell",
                "other": "Other_Languages",
            }
            mapped_language = language_mapping.get(subcategory.lower(), subcategory)
            return f"{base_path}/{mapped_language}"

        # Priority 2: Use enhanced language detection
        language = self._detect_language_from_content(content, metadata)
        return f"{base_path}/{language}"

    def _classify_concept(self, content: str, metadata: KnowledgeMetadata) -> str:
        """Classify concept content."""
        base_path = "20_Knowledge_Base/Concepts"

        # Priority 1: Check for explicit subcategory in metadata
        subcategory = getattr(metadata, "subcategory", None)
        if subcategory:
            # Map subcategory to proper directory name
            subcategory_mapping = {
                "development_patterns": "Development_Patterns",
                "ai_fundamentals": "AI_Fundamentals",
                "best_practices": "Best_Practices",
                "llm_architecture": "LLM_Architecture",
            }
            mapped_subcategory = subcategory_mapping.get(
                subcategory.lower(), subcategory
            )
            return f"{base_path}/{mapped_subcategory}"

        # Priority 2: Check tags for domain-specific classification
        domain = self._extract_domain_from_tags(metadata.tags)
        if domain:
            return f"{base_path}/{domain}"

        content_lower = content.lower()

        # For simple test cases, place in base directory
        if len(content) < 200 and not any(
            term in content_lower
            for term in [
                "artificial intelligence",
                "machine learning",
                "neural network",
                "transformer",
                "attention",
                "llm",
                "gpt",
                "pattern",
                "methodology",
                "framework",
                "architecture",
                "design",
                "best practice",
                "guideline",
            ]
        ):
            return base_path

        # AI fundamentals
        if any(
            term in content_lower
            for term in [
                "artificial intelligence",
                "machine learning",
                "neural network",
                "ai basics",
            ]
        ):
            return f"{base_path}/AI_Fundamentals"

        # LLM architecture
        if any(
            term in content_lower
            for term in [
                "large language model",
                "transformer",
                "attention",
                "llm",
                "gpt",
            ]
        ):
            return f"{base_path}/LLM_Architecture"

        # Development patterns
        if any(
            term in content_lower
            for term in [
                "pattern",
                "methodology",
                "framework",
                "architecture",
                "design",
            ]
        ):
            return f"{base_path}/Development_Patterns"

        # Best practices
        if any(
            term in content_lower
            for term in ["best practice", "guideline", "standard", "convention"]
        ):
            return f"{base_path}/Best_Practices"

        # Default to base directory for compatibility
        return base_path

    def _classify_resource(self, content: str, metadata: KnowledgeMetadata) -> str:
        """Classify resource content."""
        base_path = "20_Knowledge_Base/Resources"

        # Priority 1: Check for explicit subcategory in metadata
        subcategory = getattr(metadata, "subcategory", None)
        if subcategory:
            subcategory_mapping = {
                "documentation": "Documentation",
                "tutorials": "Tutorials",
                "research_papers": "Research_Papers",
                "tools_and_services": "Tools_And_Services",
            }
            mapped_subcategory = subcategory_mapping.get(
                subcategory.lower(), subcategory
            )
            return f"{base_path}/{mapped_subcategory}"

        content_lower = content.lower()

        # For simple test cases, place in base directory
        if len(content) < 200 and not any(
            term in content_lower
            for term in [
                "documentation",
                "docs",
                "api reference",
                "manual",
                "tutorial",
                "guide",
                "how to",
                "paper",
                "research",
                "arxiv",
                "tool",
                "service",
            ]
        ):
            return base_path

        # Documentation
        if any(
            term in content_lower
            for term in ["documentation", "docs", "api reference", "manual"]
        ):
            return f"{base_path}/Documentation"

        # Tutorials
        if any(
            term in content_lower
            for term in ["tutorial", "guide", "how to", "step by step", "walkthrough"]
        ):
            return f"{base_path}/Tutorials"

        # Research papers
        if any(
            term in content_lower
            for term in ["paper", "research", "arxiv", "journal", "study"]
        ):
            return f"{base_path}/Research_Papers"

        # Tools and services
        if any(
            term in content_lower
            for term in ["tool", "service", "platform", "software", "application"]
        ):
            return f"{base_path}/Tools_And_Services"

        # Default to base directory for compatibility
        return base_path

    def _classify_command(self, content: str, metadata: KnowledgeMetadata) -> str:
        """Classify command content to system _commands directory."""
        base_path = "_commands"

        # Priority 1: Check for explicit subcategory in metadata
        subcategory = getattr(metadata, "subcategory", None)
        if subcategory:
            subcategory_mapping = {
                "slash_commands": "Slash_Commands",
                "cli_tools": "CLI_Tools",
                "automation": "Automation",
                "scripts": "Scripts",
            }
            mapped_subcategory = subcategory_mapping.get(
                subcategory.lower(), subcategory
            )
            return f"{base_path}/{mapped_subcategory}"

        content_lower = content.lower()

        # Detect slash commands (executable files or documentation)
        if any(
            term in content_lower
            for term in ["#!/bin/bash", "スラッシュコマンド", "slash command", "/ckc-"]
        ) or content.strip().startswith("#!/"):
            return f"{base_path}/Slash_Commands"

        # CLI tools and CKC commands
        if any(
            term in content_lower
            for term in ["uv run ckc", "cli", "command line", "コマンドライン"]
        ):
            return f"{base_path}/CLI_Tools"

        # Automation scripts
        if any(
            term in content_lower
            for term in ["automation", "script", "自動化", "batch", "pipeline"]
        ):
            return f"{base_path}/Automation"

        # Default to Scripts subdirectory
        return f"{base_path}/Scripts"

    def _classify_for_project(
        self, metadata: KnowledgeMetadata, project_name: str
    ) -> str | None:
        """Classify content for specific project."""
        # When project_name is explicitly provided, always use project directory
        # This overrides tag-based classification for project-specific content
        if project_name:
            return f"10_Projects/{project_name}"

        return None

    def _classify_by_content_analysis(
        self, content: str, metadata: KnowledgeMetadata
    ) -> str:
        """Classify based on content analysis when no specific tags."""
        content_lower = content.lower()

        # Check for command patterns (highest priority for executable content)
        if any(
            pattern in content_lower
            for pattern in [
                "#!/bin/bash",
                "#!/bin/sh",
                "uv run ckc",
                "/ckc-",
                "スラッシュコマンド",
                "slash command",
                "cli command",
                "automation",
                "自動化",
            ]
        ) or content.strip().startswith("#!"):
            return self._classify_command(content, metadata)

        # Check for code patterns
        if any(
            pattern in content for pattern in ["```", "def ", "function ", "class "]
        ):
            return self._classify_code(content, metadata)

        # Check for concept patterns first (before prompt patterns)
        if any(
            pattern in content_lower
            for pattern in ["concept", "theory", "principle", "understand", "explain"]
        ):
            return self._classify_concept(content, metadata)

        # Check for prompt patterns (but be more specific)
        if any(
            pattern in content_lower
            for pattern in [
                "prompt instruction",
                "claude prompt",
                "system message",
                "user instruction",
            ]
        ):
            return self._classify_prompt(content, metadata)

        # Default to Catalyst Lab for experimental content
        return "00_Catalyst_Lab"

    def _extract_domain_from_tags(self, tags: list[str]) -> str | None:
        """Extract domain from tags for concept classification."""
        domain_mapping = {
            "api": "API_Design",
            "architecture": "Software_Architecture",
            "design": "Design_Patterns",
            "development": "Development_Practices",
            "ai": "AI_Fundamentals",
            "machine learning": "AI_Fundamentals",
            "ml": "AI_Fundamentals",
            "llm": "LLM_Architecture",
            "transformer": "LLM_Architecture",
            "security": "Security",
            "testing": "Testing",
            "deployment": "Deployment",
            "devops": "DevOps",
        }

        for tag in tags:
            tag_lower = tag.lower()
            if tag_lower in domain_mapping:
                return domain_mapping[tag_lower]

        return None

    def _detect_language_from_content(
        self, content: str, metadata: KnowledgeMetadata
    ) -> str:
        """Detect programming language from content and metadata."""
        # Check metadata tags first
        for tag in metadata.tags:
            tag_lower = tag.lower()
            if tag_lower in [
                "python",
                "javascript",
                "bash",
                "shell",
                "java",
                "cpp",
                "c++",
                "go",
                "rust",
            ]:
                # Special cases for consistent naming
                if tag_lower == "cpp" or tag_lower == "c++":
                    return "C++"
                elif tag_lower == "javascript":
                    return "JavaScript"  # 大文字S
                else:
                    return tag_lower.title()

        # Check content for language indicators
        content_lower = content.lower()

        if any(
            pattern in content
            for pattern in ["#!/bin/bash", "#!/bin/sh", "git ", "bash", "$"]
        ):
            return "Bash"
        elif any(
            keyword in content_lower
            for keyword in ["def ", "import ", "from ", "python"]
        ):
            return "Python"
        elif any(
            keyword in content_lower
            for keyword in ["function ", "const ", "let ", "var ", "javascript"]
        ):
            return "JavaScript"
        elif any(
            keyword in content_lower
            for keyword in ["public class", "import java", "java"]
        ):
            return "Java"
        elif any(
            keyword in content_lower for keyword in ["#include", "int main", "c++"]
        ):
            return "C++"
        elif any(
            keyword in content_lower for keyword in ["package main", "func ", "golang"]
        ):
            return "Go"
        elif any(keyword in content_lower for keyword in ["fn ", "use ", "rust"]):
            return "Rust"

        return "Other_Languages"


class HybridObsidianVaultManager(ObsidianVaultManager):
    """Enhanced Obsidian vault manager with hybrid structure support."""

    def __init__(
        self, vault_path: Path, metadata_manager: MetadataManager, config: CKCConfig
    ):
        super().__init__(vault_path, metadata_manager)
        self.config = config
        self.hybrid_config = config.hybrid_structure
        self.structure_manager = StructureManager(self.hybrid_config)
        self.classifier = KnowledgeClassifier(self.hybrid_config)

        # Override vault structure if hybrid is enabled
        if self.hybrid_config.enabled:
            self.vault_structure = self.structure_manager.get_vault_structure()

    def initialize_vault(self) -> bool:
        """Initialize vault with hybrid structure support."""
        if self.hybrid_config.enabled:
            return self._initialize_hybrid_vault()
        else:
            return super().initialize_vault()

    def _initialize_hybrid_vault(self) -> bool:
        """Initialize vault with hybrid structure."""
        try:
            # Create vault directory
            self.vault_path.mkdir(parents=True, exist_ok=True)

            # Create directory structure
            for dir_path, description in self.vault_structure.items():
                full_path = self.vault_path / dir_path
                full_path.mkdir(parents=True, exist_ok=True)

                # Create README.md for each directory
                readme_path = full_path / "README.md"
                if not readme_path.exists():
                    readme_content = self._generate_directory_readme(
                        dir_path, description
                    )
                    readme_path.write_text(readme_content, encoding="utf-8")

            # Deploy automation assets if enabled
            if self.hybrid_config.auto_enhancement:
                self._deploy_automation_assets()

            # Create vault configuration
            self._create_obsidian_config()

            # Validate structure if enabled
            if self.hybrid_config.structure_validation:
                validation_result = self._validate_structure()
                if not validation_result:
                    print("⚠️ Structure validation warnings detected")

            print(f"✅ Hybrid vault initialized at: {self.vault_path}")
            return True

        except Exception as e:
            print(f"❌ Error initializing hybrid vault: {e}")
            return False

    def _determine_target_path(
        self, metadata: KnowledgeMetadata, source_path: Path, project_name: str | None
    ) -> Path:
        """Determine target path with hybrid classification support."""

        if self.hybrid_config.enabled:
            # Use hybrid classifier
            content = source_path.read_text(encoding="utf-8")
            target_dir = self.classifier.classify_content(
                content, metadata, source_path, project_name
            )

            # Generate filename
            filename = self._generate_enhanced_filename(metadata, source_path)

            return self.vault_path / target_dir / filename
        else:
            # Use legacy logic
            return super()._determine_target_path(metadata, source_path, project_name)

    def _generate_enhanced_filename(
        self, metadata: KnowledgeMetadata, source_path: Path
    ) -> str:
        """Generate enhanced filename for hybrid structure."""
        # Use title as base, fallback to original filename
        if metadata.title and metadata.title != "Untitled":
            base_name = metadata.title
        else:
            base_name = source_path.stem

        # Clean filename (remove special characters)

        clean_name = re.sub(r'[<>:"/\\|?*]', "_", base_name)

        # Add date prefix for better organization
        date_prefix = metadata.created.strftime("%Y%m%d")

        # Add version if specified
        version_suffix = ""
        if metadata.version and metadata.version != "1.0":
            version_suffix = f"_v{metadata.version}"

        # Ensure .md extension (avoid double extension)
        filename = f"{date_prefix}_{clean_name}{version_suffix}"
        if not filename.endswith(".md"):
            filename += ".md"

        return filename

    def _generate_directory_readme(self, dir_path: str, description: str) -> str:
        """Generate README content for directory."""
        dir_name = dir_path.split("/")[-1]
        classification = self.hybrid_config.classify_directory(dir_name)

        # Determine tier emoji
        tier_emoji = {
            DirectoryTier.SYSTEM: "⚙️",
            DirectoryTier.CORE: "📂",
            DirectoryTier.AUXILIARY: "📋",
        }

        emoji = tier_emoji.get(classification.tier, "📁")

        content = f"""# {emoji} {dir_name}

## 📋 概要
{description}

## 🎯 目的
{classification.purpose}

## 📊 分類情報
- **層**: {classification.tier.value}
- **プレフィックス**: {classification.prefix or "なし"}
- **番号**: {classification.number or "なし"}
- **自動整理**: {"有効" if classification.auto_organization else "無効"}

## 📝 使用方法
このディレクトリは{classification.tier.value}層に分類されており、{classification.purpose.lower()}に使用されます。

"""

        # Add specific guidance based on directory type
        if "Knowledge_Base" in dir_path:
            content += """## 📚 Knowledge Base 構造
このディレクトリは知識ベースの一部として、以下の構造に従って整理されています：

- **Prompts/**: プロンプト関連知識
- **Code_Snippets/**: コードスニペット集
- **Concepts/**: AI・LLM関連概念
- **Resources/**: 学習リソース・参考資料

"""

        if classification.tier == DirectoryTier.SYSTEM:
            content += """## ⚠️ システムディレクトリ
このディレクトリはシステム管理用です。手動でのファイル配置は推奨されません。

"""

        content += f"""---
*このREADMEは自動生成されました - {dir_path}*
"""

        return content

    def _deploy_automation_assets(self) -> None:
        """Deploy automation scripts and templates."""
        scripts_dir = self.vault_path / "_scripts"
        if scripts_dir.exists():
            # Could deploy validation and enhancement scripts here
            pass

    def _validate_structure(self) -> bool:
        """Validate vault structure integrity."""
        issues = []

        # Check all expected directories exist
        for dir_path in self.vault_structure.keys():
            full_path = self.vault_path / dir_path
            if not full_path.exists():
                issues.append(f"Missing directory: {dir_path}")
            elif not full_path.is_dir():
                issues.append(f"Not a directory: {dir_path}")

        # Check for unexpected directories in root
        if self.hybrid_config.structure_validation:
            expected_dirs = set(self.vault_structure.keys())
            actual_dirs = {d.name for d in self.vault_path.iterdir() if d.is_dir()}

            unexpected = actual_dirs - expected_dirs - {".obsidian"}
            if unexpected:
                issues.append(f"Unexpected directories: {', '.join(unexpected)}")

        if issues:
            print("⚠️ Structure validation issues:")
            for issue in issues:
                print(f"  - {issue}")
            return False

        return True

    def get_structure_info(self) -> dict[str, Any]:
        """Get information about current structure."""
        return {
            "hybrid_enabled": self.hybrid_config.enabled,
            "numbering_system": self.hybrid_config.numbering_system.value,
            "structure_version": self.hybrid_config.structure_version,
            "auto_classification": self.hybrid_config.auto_classification,
            "directory_count": len(self.vault_structure),
            "vault_stats": self.get_vault_stats() if self.vault_path.exists() else {},
        }
